@@ -301,7 +301,6 @@ static void sync_frame_out_cb(void *user_data, const ubit_t *bits, unsigned int
 
 	mgcp_send(endp, 1, NULL, msg, &conn_dst->u.rtp, &conn_dst->u.rtp);
 
-	msgb_free(msg);
 	return;
 skip:
 	rate_ctr_inc(rate_ctr_group_get_ctr(rate_ctrs, E1_I460_TRAU_RX_FAIL_CTR));
@@ -325,7 +324,7 @@ static void e1_send_ts_frame(struct e1inp_ts *ts, struct mgcp_trunk *trunk)
 				     msgb_length(msg) > DEBUG_BYTES_MAX ? DEBUG_BYTES_MAX : msgb_length(msg)));
 #endif
 	/* Hand data over to the E1 stack */
-	msgb_enqueue(&ts->raw.tx_queue, msg);
+	e1inp_ts_send_raw(ts, msg);
 	return;
 }
 
@@ -466,11 +465,21 @@ static enum osmo_trau_frame_type determine_trau_fr_type(char *sdp_subtype_name,
 {
 	if (strcmp(sdp_subtype_name, "GSM") == 0)
 		return OSMO_TRAU16_FT_FR;
-	else if (strcmp(sdp_subtype_name, "GSM-EFR") == 0)
+
+	if (strcmp(sdp_subtype_name, "GSM-EFR") == 0)
 		return OSMO_TRAU16_FT_EFR;
-	else if (strcmp(sdp_subtype_name, "GSM-HR-08") == 0)
-		return OSMO_TRAU16_FT_HR;
-	else if (strcmp(sdp_subtype_name, "AMR") == 0) {
+
+	if (strcmp(sdp_subtype_name, "GSM-HR-08") == 0) {
+		if (i460_rate == OSMO_I460_RATE_16k)
+			return OSMO_TRAU16_FT_HR;
+		if (i460_rate == OSMO_I460_RATE_8k)
+			return OSMO_TRAU8_SPEECH;
+		LOGPENDP(endp, DE1, LOGL_ERROR,
+			 "E1-TRAU-TX: unsupported or illegal I.460 rate for HR\n");
+		return OSMO_TRAU_FT_NONE;
+	}
+
+	if (strcmp(sdp_subtype_name, "AMR") == 0) {
 		if (i460_rate == OSMO_I460_RATE_8k) {
 			switch (amr_ft) {
 			case AMR_4_75:
@@ -488,11 +497,11 @@ static enum osmo_trau_frame_type determine_trau_fr_type(char *sdp_subtype_name,
 			}
 		}
 		return OSMO_TRAU16_FT_AMR;
-	} else {
-		LOGPENDP(endp, DE1, LOGL_ERROR, "E1-TRAU-TX: unsupported or illegal codec subtype name: %s\n",
-			 sdp_subtype_name);
-		return OSMO_TRAU_FT_NONE;
 	}
+
+	LOGPENDP(endp, DE1, LOGL_ERROR, "E1-TRAU-TX: unsupported or illegal codec subtype name: %s\n",
+		 sdp_subtype_name);
+	return OSMO_TRAU_FT_NONE;
 }
 
 /* Determine a suitable TRAU frame type for a given codec */
@@ -501,11 +510,21 @@ static enum osmo_tray_sync_pat_id determine_trau_sync_pat(char *sdp_subtype_name
 {
 	if (strcmp(sdp_subtype_name, "GSM") == 0)
 		return OSMO_TRAU_SYNCP_16_FR_EFR;
-	else if (strcmp(sdp_subtype_name, "GSM-EFR") == 0)
+
+	if (strcmp(sdp_subtype_name, "GSM-EFR") == 0)
 		return OSMO_TRAU_SYNCP_16_FR_EFR;
-	else if (strcmp(sdp_subtype_name, "GSM-HR-08") == 0)
-		return OSMO_TRAU_SYNCP_8_HR;
-	else if (strcmp(sdp_subtype_name, "AMR") == 0) {
+
+	if (strcmp(sdp_subtype_name, "GSM-HR-08") == 0) {
+		if (i460_rate == OSMO_I460_RATE_16k)
+			return OSMO_TRAU_SYNCP_16_FR_EFR;
+		if (i460_rate == OSMO_I460_RATE_8k)
+			return OSMO_TRAU_SYNCP_8_HR;
+		LOGPENDP(endp, DE1, LOGL_ERROR,
+			 "E1-TRAU-TX: unsupported or illegal I.460 rate for HR\n");
+		return OSMO_TRAU_SYNCP_16_FR_EFR;
+	}
+
+	if (strcmp(sdp_subtype_name, "AMR") == 0) {
 		if (i460_rate == OSMO_I460_RATE_8k) {
 			switch (amr_ft) {
 			case AMR_4_75:
@@ -523,11 +542,11 @@ static enum osmo_tray_sync_pat_id determine_trau_sync_pat(char *sdp_subtype_name
 			}
 		}
 		return OSMO_TRAU_SYNCP_16_FR_EFR;
-	} else {
-		LOGPENDP(endp, DE1, LOGL_ERROR, "E1-TRAU-TX: unsupported or illegal codec subtype name: %s\n",
-			 sdp_subtype_name);
-		return OSMO_TRAU_SYNCP_16_FR_EFR;
 	}
+
+	LOGPENDP(endp, DE1, LOGL_ERROR, "E1-TRAU-TX: unsupported or illegal codec subtype name: %s\n",
+		 sdp_subtype_name);
+	return OSMO_TRAU_SYNCP_16_FR_EFR;
 }
 
 /* Find out if a given TRAU frame type is AMR */
